import enum
from dataclasses import dataclass
from enum import auto
from typing import Dict


class PaymentMethodType(enum.Enum):
    REDIRECT = auto()
    FORM = auto()


class PaymentMethod:
    method: str


@dataclass
class PaymentRedirect(PaymentMethod):
    method = "redirect"
    url: str


@dataclass
class PaymentForm(PaymentMethod):
    method = "form"
    action: str
    fields: Dict[str, str]



from decimal import Decimal as D
from typing import List, Optional
from uuid import UUID

from django.urls import reverse
from django.utils.translation import get_language
from ipware.ip import get_client_ip

from django_pay2.exceptions import CreatePaymentError
from django_pay2.payment_methods import PaymentMethodType, PaymentRedirect
from django_pay2.providers import register
from django_pay2.providers.base import PaymentSystem
from django_pay2.settings import payment_settings

from .api import TinkoffApiError, TinkoffOrderItem
from .services import get_tinkoff_api


@register
class Tinkoff(PaymentSystem):
    name = "tinkoff"
    verbose_name = "Тинькофф"
    method_type = PaymentMethodType.REDIRECT

    def generate_payment_method(
        self,
        payment_id: UUID,
        amount: D,
        request,
        desc,
        items: List[TinkoffOrderItem],
        client_email: Optional[str] = None,
        client_phone: Optional[str] = None,
        **kwargs
    ):
        try:
            api = get_tinkoff_api()
            notification_url = reverse("django_pay2:tinkoff:notify")
            success_url = payment_settings.TINKOFF.return_url
            fail_url = payment_settings.TINKOFF.fail_url
            result = api.init_payment(
                amount,
                str(payment_id),
                get_client_ip(request)[0],
                desc,
                get_language(),
                notification_url,
                success_url,
                fail_url,
                items=items,
                client_email=client_email,
                client_phone=client_phone,
                email_company=payment_settings.TINKOFF.email_company,
                taxation=payment_settings.TINKOFF.taxation,
            )
            return PaymentRedirect(result.payment_url)
        except TinkoffApiError as exc:
            raise CreatePaymentError(str(exc))



from django.apps import AppConfig


class DjangoPayConfig(AppConfig):
    name = "django_pay2"
    verbose_name = "Платежи"

    def ready(self) -> None:
        import django_pay2.providers.coinpayments  # noqa:
        import django_pay2.providers.free_kassa  # noqa:
        import django_pay2.providers.payeer  # noqa:
        import django_pay2.providers.perfect_money  # noqa:
        import django_pay2.providers.qiwi  # noqa:
        import django_pay2.providers.qiwi_kassa  # noqa:
        import django_pay2.providers.sberbank  # noqa:
        import django_pay2.providers.tinkoff  # noqa:



from decimal import Decimal as D
from uuid import UUID

from django_pay2.payment_methods import PaymentMethodType
from django_pay2.providers import register
from django_pay2.providers.base import PaymentSystem

from .api import get_api


@register
class PerfectMoney(PaymentSystem):
    name = "perfect_money"
    verbose_name = "PerfectMoney"
    method_type = PaymentMethodType.FORM

    def generate_payment_method(
        self, payment_id: UUID, amount: D, request, currency, **kwargs
    ):
        api = get_api()
        return api.generate_payment_method(request, amount, currency, payment_id)



from decimal import Decimal as D
from uuid import UUID

from django_pay2.payment_methods import PaymentMethodType
from django_pay2.providers import register
from django_pay2.providers.base import PaymentSystem

from .api import get_api


@register
class FreeKassa(PaymentSystem):
    name = "free_kassa"
    verbose_name = "Free Kassa"
    method_type = PaymentMethodType.FORM

    def generate_payment_method(self, payment_id: UUID, amount: D, **kwargs):
        api = get_api()
        return api.generate_payment_method(amount=amount, order_id=payment_id)



# Generated by Django 3.2.7 on 2021-11-27 09:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_pay2', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='payment',
            name='payment_system',
            field=models.CharField(default='qiwi_kassa', max_length=50, verbose_name='Код платежной системы'),
            preserve_default=False,
        ),
    ]



from decimal import Decimal as D
from uuid import UUID

from django_pay2.exceptions import CreatePaymentError
from django_pay2.payment_methods import PaymentMethodType
from django_pay2.providers import register
from django_pay2.providers.base import PaymentSystem

from .exceptions import PayeerError
from .functions import get_payeer_api


@register
class Payeer(PaymentSystem):
    name = "payeer"
    verbose_name = "Payeer"
    method_type = PaymentMethodType.REDIRECT

    def generate_payment_method(
        self, payment_id: UUID, amount: D, currency, desc, **kwargs
    ):
        try:
            api = get_payeer_api()
            return api.create_payment(str(payment_id), amount, currency, desc)
        except PayeerError as exc:
            raise CreatePaymentError(str(exc))



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from django_pay2.providers.base import PaymentSystem


_PAYMENT_SYSTEMS = {}


def register(payment_system):
    assert (
        payment_system.name
        and payment_system.verbose_name
        and payment_system.method_type
    ), f"Define `name`, `verbose_name` and `method_type` attributes for {payment_system}"

    _PAYMENT_SYSTEMS[payment_system.name] = payment_system()

    return payment_system


def get_payment_system(name: str) -> "PaymentSystem":
    return _PAYMENT_SYSTEMS[name]



from decimal import Decimal as D
from typing import Any

from django_pay2.providers import get_payment_system


def create_payment(amount: D, receiver: Any, payment_system_name: str, **kwargs):
    try:
        payment_system = get_payment_system(payment_system_name)
    except KeyError:
        raise AssertionError(
            f"Payment system with name={payment_system_name} does not exist"
        )

    return payment_system.create_payment(receiver=receiver, amount=amount, **kwargs)



from django_pay2.payment_methods import PaymentMethodType
from django_pay2.providers import register
from django_pay2.providers.base import PaymentSystem

from .api import get_api


@register
class CoinPayments(PaymentSystem):
    name = "coinpayments"
    verbose_name = "CoinPayments"
    method_type = PaymentMethodType.REDIRECT

    def generate_payment_method(
        self, payment_id, amount, currency, buyer_email, request, **kwargs
    ):
        api = get_api()
        return api.generate_payment_method(
            request, amount, currency, payment_id, buyer_email
        )



from datetime import datetime, timedelta
from decimal import Decimal as D
from uuid import UUID

from django_pay2.payment_methods import PaymentMethodType
from django_pay2.providers import register
from django_pay2.providers.base import PaymentSystem

from .api import get_api


@register
class QiwiKassa(PaymentSystem):
    name = "qiwi_kassa"
    verbose_name = "Qiwi Kassa"
    method_type = PaymentMethodType.REDIRECT

    def generate_payment_method(
        self, payment_id: UUID, amount: D, currency, expirate_at=None, **kwargs
    ):
        expirate_at = expirate_at or datetime.utcnow() + timedelta(hours=12)

        api = get_api()
        return api.generate_payment_method(amount, currency, expirate_at, payment_id)



from decimal import Decimal as D
from typing import Optional
from uuid import UUID

from django_pay2.payment_methods import PaymentMethodType
from django_pay2.providers import register
from django_pay2.providers.base import PaymentSystem
from django_pay2.settings import payment_settings

from .api import SberbankApi


def get_api() -> SberbankApi:
    return SberbankApi(
        username=payment_settings.SBERBANK.username,
        password=payment_settings.SBERBANK.password,
    )


@register
class Sberbank(PaymentSystem):
    name = "sberbank"
    verbose_name = "Сбербанк"
    method_type = PaymentMethodType.REDIRECT

    def generate_payment_method(
        self,
        payment_id: UUID,
        amount: D,
        desc: str,
        return_url: Optional[str] = None,
        fail_url: Optional[str] = None,
        page_view: Optional[str] = None,
        phone: Optional[str] = None,
        **kwargs
    ):
        return_url = return_url or payment_settings.SBERBANK.return_url
        fail_url = fail_url or payment_settings.SBERBANK.fail_url
        api = get_api()
        return api.register_payment(
            order_num=str(payment_id),
            amount=str(amount),
            return_url=return_url,
            fail_url=fail_url,
            description=desc,
            page_view=page_view,
            phone=phone,
        )



from decimal import Decimal as D
from typing import Any
from uuid import UUID

from django.urls import reverse

from django_pay2.models import Payment
from django_pay2.payment_methods import (
    PaymentForm,
    PaymentMethod,
    PaymentMethodType,
    PaymentRedirect,
)
from django_pay2.settings import payment_settings


class PaymentSystem:
    name: str
    verbose_name: str
    method_type: PaymentMethodType

    def create_payment(self, receiver: Any, amount: D, **kwargs) -> PaymentMethod:
        payment = Payment.objects.create(
            receiver=receiver, amount=amount, payment_system=self.name
        )

        if payment_settings.DEBUG_MODE:
            return self.generate_debug_payment_method(
                payment_id=payment.id, amount=amount
            )
        return self.generate_payment_method(
            payment_id=payment.id, amount=amount, **kwargs
        )

    def generate_payment_method(
        self, payment_id: UUID, amount: D, **kwargs
    ) -> PaymentMethod:
        raise NotImplementedError

    def generate_debug_payment_method(
        self, payment_id: UUID, amount: D, **kwargs
    ) -> PaymentMethod:
        if self.method_type == PaymentMethodType.REDIRECT:
            return self._generate_redirect_debug_payment(payment_id=payment_id)
        elif self.method_type == PaymentMethodType.FORM:
            return self._generate_form_debug_payment(
                payment_id=payment_id, amount=amount
            )
        raise AssertionError(f"Unexpected method_type={self.method_type}")

    def _generate_redirect_debug_payment(self, payment_id: UUID) -> PaymentRedirect:
        return PaymentRedirect(
            url=reverse("django_pay2:debug_payment", args=[payment_id])
        )

    def _generate_form_debug_payment(self, payment_id: UUID, amount: D) -> PaymentForm:
        return PaymentForm(
            action=reverse("django_pay2:debug_payment", args=[payment_id]),
            fields={"amount": str(amount)},
        )
